local entity_filter = {}
local down_binner_target_map = {}
local quality_level = {}

for _, quality in pairs(prototypes.quality) do
  if not quality.hidden then
    table.insert(entity_filter, { filter = "name", name = "down-binner-" .. quality.name })
    down_binner_target_map["down-binner-" .. quality.name] = quality.name
    quality_level[quality.name] = quality.level
  end
end

local exclude_list = {
  ["tank"] = true,
  ["spidertron"] = true,
  ["modular-armor"] = true,
  ["power-armor"] = true,
  ["power-armor-mk2"] = true,
  ["mech-armor"] = true
}

local function on_init()
  storage.down_binners = {}
end

local function on_built(evt)
  storage.down_binners[evt.entity.unit_number] = evt.entity
end

local function down_bin()
  for unit_number, entity in pairs(storage.down_binners) do

    if (not entity or not entity.valid) then
      table.remove(storage.down_binners, unit_number)
      goto next_entity
    end

    local down_binner_target = down_binner_target_map[entity.name]
    local down_binner_target_level = quality_level[down_binner_target]

    local inventory = entity.get_inventory(defines.inventory.chest)
    local contents = inventory.get_contents()

    for _, content in pairs(contents) do

      if exclude_list[content.name] then
        goto next_content
      end

      if not quality_level[content.quality] then
        goto next_content
      end

      if down_binner_target_level < quality_level[content.quality] then
        inventory.remove(content)
        content.quality = down_binner_target
        inventory.insert(content)
      end

      :: next_content ::
    end

    :: next_entity ::
  end
end

script.on_init(on_init)
-- script.on_configuration_changed()

script.on_event(defines.events.on_built_entity, on_built, entity_filter)
script.on_event(defines.events.on_robot_built_entity, on_built, entity_filter)
script.on_event(defines.events.script_raised_built, on_built, entity_filter)
script.on_event(defines.events.script_raised_revive, on_built, entity_filter)
if script.active_mods["space-age"] then
  script.on_event(defines.events.on_space_platform_built_entity, on_built, entity_filter)
end

script.on_nth_tick(1, down_bin)
